/**
 * Function that tracks the amount of opt-out for analytics data in Google
 * Analytics.
 */
function trackShareData() {
    gtag('event', 'click', {
        'event_category': 'share-data',
        'event_label': 'opt-out'
    });
}

$(function () {
    /**
     * Track clicks on an outbound link in Google Analytics. This tracking is
     * added to all links with class 'outbound' and takes the href string and
     * uses that URL string as the event label.
     */
    $('a.outbound').click(function() {
        url = $(this).attr('href');
        gtag('event', 'click', {
            'event_category': 'outbound',
            'event_label': url,
            'transport_type': 'beacon',
            'event_callback': function() {document.location = url;}
        });
    });

    /**
     * Track clicks on help icons in Google Analytics. This tracking is added
     * to all i-tags with class 'helpIcon' and takes the for string and uses
     * that id string as the event label. Doesn't track already visible help.
     */
    $('i.helpIcon').click(function() {
        if (!$(this).hasClass('active')) {
            label = $(this).attr('for');
            gtag('event', 'click', {
                'event_category': 'help-icon',
                'event_label': label
            });
        }
    });

    /**
     * Track clicks on share data's "learn more" button in Google Analytics.
     * This tracking is added to all a-tags with class 'privacy-dialog-link'.
     */
    $('a.privacy-dialog-link').click(function() {
        gtag('event', 'click', {
            'event_category': 'learn-more',
            'event_label': 'privacy-policy'
        });
    });
});