$(function() {
    var $startButton = $("#start-button"),
        $stopButton = $("#stop-button"),
        $progressBar = $("#progressbar"),
        $progressText = $("#progress-label"),
        $proceedButton = $("#proceed-button"),
        $stepText = $("#status-text"),
        $infoText = $("#info-text"),
        $settingsButton = $("#settings-button"),
        $skipCalibrationButton = $("#skip-calibration-button"),
        timer;
    $.ajaxSetup({ cache: false });
    $proceedButton.attr("formaction", location.protocol + "//" + location.host + "/stereo/");

    let doCapture = false;
    var processStarted = false;
    function calibrationStartHandler(event) {
        event.preventDefault();
        $stepText.html("Starting...");
        $skipCalibrationButton.attr("disabled", "disabled");
        $settingsButton.attr("disabled", "disabled");
        $progressText.text("");
        $progressBar.slideDown();
        $.post("/stereo/calibrate/start");
        $startButton.attr("disabled", "disabled");
        $proceedButton.attr("disabled", "disabled");
        clearInterval(timer);
        timer = setInterval(updateState, 2000);
        processStarted = true;
        doCapture = true;
    };

    function calibrationCaptureHandler(event) {
        event.preventDefault();
        $.get("/stereo/calibrate/capture").then((data) => {
            $startButton.removeAttr("disabled");
        });
        $startButton.attr("disabled", "disabled");
    };

    function resetArucoCalibrationState() {
        $startButton.unbind("click");
        $startButton.click(calibrationStartHandler);
    }

    $startButton.click(calibrationStartHandler);

    $stopButton.click(function(e) {
        e.preventDefault();
        $.post("/stereo/calibrate/stop");
        almostDone = false;
        clearInterval(timer);
        clearWarnings();
        if (canvas) {
            canvas.clear();
        }
        resetArucoCalibrationState()
        $stopButton.attr("disabled", "disabled");
        $stepText.html("Aborting... This might take a while.");
        if ($progressBar.css('display') === 'none') {
            $progressBar.show();
        }
        $progressBar.progressbar("value", false);
        timer = setInterval(updateState, 5000);
        processStarted = false;
        $skipCalibrationButton.attr("disabled", "disabled");
    });

    var almostDone = false;
    let canvas = new fabric.StaticCanvas("video_canvas");

    $progressBar.progressbar({
        value: false,
        change: function() {
            var cur_val = $progressBar.progressbar( "value" );
            if (cur_val === false) {
                $progressText.text("");
            } else {
                $progressText.text(cur_val  + "%" );
            }
        },
      complete: function() {
        $progressText.text( "Complete!" );
      }
    }).hide();

    function updateUI() {
        $.getJSON("/stereo/calibration.json", function(calib) {
            if (params.device.calibration.has_calibration && calib.step === 0) {
                $proceedButton.removeAttr("disabled");
            } else {
                $proceedButton.attr("disabled", "disabled");
            }
            let arucoCalibration = calib.hasOwnProperty("accepted")
            almostDone = almostDone || calib.step > 0;
            if ( calib.step === 0 ) {
                if (almostDone) {
                    $stopButton.attr("disabled", "disabled");
                    $stepText.html("Calibration completed!");
                    $settingsButton.removeAttr("disabled");
                    $skipCalibrationButton.attr("disabled", "disabled");
                    if (arucoCalibration) {
                        canvas.clear();
                        resetArucoCalibrationState()
                    }
                    $stepText.html("Calibration completed!");
                    processStarted = false;
                } else if (!processStarted) {
                    $startButton.removeAttr("disabled");
                    $stopButton.attr("disabled", "disabled");
                    if ( params.device.calibration.has_calibration ) {
                        $settingsButton.removeAttr("disabled");
                        $stepText.html("Camera is calibrated - Ready for recalibration.");
                    } else {
                        $skipCalibrationButton.removeAttr("disabled");
                        $stepText.html("This camera is not calibrated - Ready for calibration.");
                    }
                }
                $progressBar.slideUp();
            } else if ( calib.step == 1 ) {
                if ($progressBar.css('display') === 'none') {
                    $progressBar.show();
                }
                $stopButton.removeAttr("disabled");
                var progress = Math.min(50, calib.points / calib.target_points * 50);
                $progressBar.progressbar("value", Math.round(progress));
                if (arucoCalibration) {
                    if (doCapture) {
                        $.get("/stereo/calibrate/capture").then((data) => {
                            $startButton.removeAttr("disabled");
                        });
                        $startButton.attr("disabled", "disabled");
                        $startButton.unbind("click");
                        $startButton.click(calibrationCaptureHandler);
                        doCapture = false;
                    }
                    drawMarkers(calib.accepted, calib.distances.concat(calib.duplicates));
                    numMarkers = calib.accepted.length
                    $stepText.html("Collecting video data... Please make sure there are 4 unique markers visible underneath the camera.");
                    if (calib.frame_time != 0) {
                        clearWarnings();
                        if (numMarkers < 4 && calib.distances.length == 0 && calib.duplicates.length == 0) {
                            document.getElementById("missing-marker").textContent = "Not all markers were detected, found " + numMarkers + " markers, expected 4";
                        }
                        if (calib.distances && calib.distances.length > 0) {
                            document.getElementById("distance-error").textContent = "Markers " + calib.distances.map(x => x[0]) + " are placed too close together.";
                        }
                        if (calib.duplicates && calib.duplicates.length > 0) {
                            document.getElementById("duplicate-error").textContent = "Duplicate markers found. Please use 4 unique markers and try again";
                        }
                    }
                } else {
                    $startButton.attr("disabled", "disabled");
                    $stepText.html("Collecting video data... Please make sure there are people moving beneath the camera.");
                }
            } else if ( calib.step == 2 ){
                drawMarkers(calib.accepted, []);
                clearWarnings();
                if ($progressBar.css('display') === 'none') {
                    $progressBar.show();
                }
                $progressBar.progressbar("value", false);
                $stopButton.removeAttr("disabled");
                $startButton.attr("disabled", "disabled");
                $stepText.html("Processing the collected video data...");
            } else if( calib.step == 3 ) {
                if (arucoCalibration) {
                    $progressBar.progressbar("value", false);
                } else {
                    canvas.clear();
                }
                $stopButton.removeAttr("disabled");
                $startButton.attr("disabled", "disabled");
                $stepText.html("Calibration complete, applying new calibration.");
            } else if( calib.step == 4 ) {
                // We will probably never come here since the camera restarts
                // and gets mode = "off" when complete.
                // That's why we have the variable almostDone.
            }
        });
    }

    function drawMarkers(accepted, rejected) {
        let iconSize = canvas.height / 9;
        let offset = iconSize / 2;
        if (typeof drawMarkers.accepted == "undefined") {
            let serializer = new XMLSerializer();
            drawMarkers.accepted = serializer.serializeToString(document.getElementById("check-svg"));
        }
        if (typeof drawMarkers.rejected == "undefined") {
            let serializer = new XMLSerializer();
            drawMarkers.rejected = serializer.serializeToString(document.getElementById("err-svg"));
        }
        let checkObjects = canvas.getObjects();
        let drawnObjectIndex = 0;
        for (i = 0; i < accepted.length; i++) {
            let center = accepted[i][1];
            let x = Math.round(canvas.width * center[0]);
            let y = Math.round(canvas.height * center[1]);
            canvas.remove(checkObjects[i]);
            fabric.loadSVGFromString(drawMarkers.accepted, (objects, options) => {
                canvas.insertAt(fabric.util.groupSVGElements(objects, options)
                               .scaleToHeight(iconSize)
                               .set({top: y - offset, left: x - offset})
                               .setCoords(), i, false);
            });
            drawnObjectIndex++;
        }
        for (i = 0; i < rejected.length; i++) {
            let objIndex = i + accepted.length;
            let x = Math.round(canvas.width * rejected[i][1]);
            let y = Math.round(canvas.height * rejected[i][2]);
            canvas.remove(checkObjects[objIndex]);
            fabric.loadSVGFromString(drawMarkers.rejected, (objects, options) => {
                canvas.insertAt(fabric.util.groupSVGElements(objects, options)
                               .scaleToHeight(iconSize)
                               .set({top: y - offset, left: x - offset})
                               .setCoords(), objIndex, false);
            });
            drawnObjectIndex++;
        }
        for (i = drawnObjectIndex; i < checkObjects.length; i++) {
            checkObjects[i].set({opacity: 0.0});
        }
        canvas.renderAll();
    }

    function clearWarnings() {
        document.getElementById("missing-marker").textContent = "";
        document.getElementById("duplicate-error").textContent = "";
        document.getElementById("distance-error").textContent = "";
    }

    function updateState() {
        $.getJSON("/stereo/params.json", function(data) {
            params = data;
            updateUI();
        });

    }
    $("#camera-button").click(function(e) {
        window.location.href = "/admin/index.shtml";
        e.preventDefault();
    });

    $settingsButton.click(function(e) {
        window.location.href = "/stereo/people-counter/settings.html";
        e.preventDefault();
    });

    $skipCalibrationButton.click(function(e) {
        window.location.href = "/stereo/people-counter/basic-settings.html";
        e.preventDefault();
    });

    updateUI();
    timer = setInterval(updateState, 5000);
    $("#left .centered").createVideo();
});
