$.fn.countingZone = function(options) {
    var self = this;
    var defaults = {
        view_only: false,
        restriction: {
            left: 0,
            right: 1,
            top: 0,
            bottom: 1,
            min_width: 0,
            min_height: 0.05,
            max_width: 1,
            max_height: 1
        }
    };
    var zone = JSON.parse(JSON.stringify(params.counter.zone));
    var installation_height = JSON.parse(JSON.stringify(params.meta.height));
    var currentArrow;
    var arrowImageData = {
        arrow_straight: {
            url: "/static/image/arrow_straight.svg",
            obj: "",
            shape: ["out", "none", "in", "none"],
            angle: 90,
        },
        arrow_left_into_straight: {
            url: "/static/image/arrow_left_into_straight.svg",
            obj: "",
            shape: ["out", "out", "in", "none"],
            angle: 90,
        },
        arrow_one_into_three: {
            url: "/static/image/arrow_one_into_three.svg",
            obj: "",
            shape: ["out", "in", "in", "in"],
            angle: 90,
        },
        arrow_straight_right_bend: {
            url: "/static/image/arrow_straight_right_bend.svg",
            obj: "",
            shape: ["out", "none", "in", "in"],
            angle: 90,
        },
        arrow_three_to_one: {
            url: "/static/image/arrow_three_to_one.svg",
            obj: "",
            shape: ["out", "out", "in", "out"],
            angle: 90,
        },
        arrow_one_into_left_right: {
            url: "/static/image/arrow_one_into_left_right.svg",
            obj: "",
            shape: ["out", "in", "none", "in"],
            angle: 90,
        },
        arrow_left_right_into_straight: {
            url: "/static/image/arrow_left_right_into_straight.svg",
            obj: "",
            shape: ["out", "in", "out", "none"],
            angle: 90,
        },
        arrow_corner_x2: {
            url: "/static/image/arrow_corner_x2.svg",
            obj: "",
            shape: ["out", "in", "in", "out"],
            angle: 45,
        },
        arrow_bend: {
            url: "/static/image/arrow_bend.svg",
            obj: "",
            shape: ["out", "in", "none", "none"],
            angle: 90,
        },
        arrow_one_into_left_right_x2: {
            url: "/static/image/arrow_one_into_left_right_x2.svg",
            obj: "",
            shape: ["out", "in", "out", "in"],
            angle: 0,
        },
    };
    loadArrowImages();

    // For convenience
    var r = options.restriction;
    var z = zone;

    function res(v) {
        return (v * 1000)|0;
    }

    function unres(v) {
        return v / 1000;
    }

    function addSlider(title, name, opts) {
        var slide_callback = opts.slide,
            $ctrl = $("<div/>").addClass("ctrlHolder"),
            $slider;
        $("<label/>").text(title).appendTo($ctrl);
        /*
        opts.slide = function(cb) {
            var valid = slide_callback.apply(this, arguments);
            if ( opts.range ) {
                var v = $slider.slider("values");
                $input.val(JSON.stringify([unres(v[0]), unres(v[1])]));
            } else {
                $input.val(JSON.stringify(unres($slider.slider("value"))));
            }
            return valid;
        };
        */
        if ( !options.view_only ) {
            $slider = $("<div/>").slider(opts||{}).addClass("uniform-slider").appendTo($ctrl);
            $ctrl.appendTo(self);
        }
        return $slider;
    }

    function loadArrowImages() {
        Object.keys(arrowImageData).forEach(function (key) {
            fabric.loadSVGFromURL(arrowImageData[key]["url"], function (objects, options) {
                var fabObj = fabric.util.groupSVGElements(objects, options);
                fabObj.set({
                    originX: "center",
                    originY: "center"
                });
                arrowImageData[key]["obj"] = fabObj;
            });
        });
    }

    options = $.extend({}, defaults, options);

    var width = options.width || 100;
    var height = options.height || 100;
    var canvas = new fabric.StaticCanvas(options.canvas, {
        width: width,
        height: height
    });

    /*
     * This variable defines all objects that are rendered on the canvas.
     * Their position on the canvas is set with the updatePositions function.
     */
    var objs = {
        dir_in_text: new fabric.Text("Direction In",{
            fontFamily:"Arial",
            fontSize:20,
            opacity:0,
            fill:"#FFF",
            shadow: 'rgba(0,0,0,1) 1px 1px',
            left:5,
            top:5,
        }),
        dir_in_up: new fabric.Text("\uf062",{
            fontFamily: 'FontAwesome',
            fontSize:20,
            opacity:0,
            fill:"#FFF",
            shadow: 'rgba(0,0,0,1) 1px 1px',
            selectable: false,
        }),
        dir_in_down: new fabric.Text("\uf063",{
            fontFamily: 'FontAwesome',
            fontSize:20,
            opacity:0,
            fill:"#FFF",
            shadow: 'rgba(0,0,0,1) 1px 1px',
            selectable: false,
        }),

        top_line_curve: new fabric.Path('M 0 20',{
            stroke:'rgba(0,100,0, 0.9)',
            strokeWidth:2,
            selectable:false
        }),

        bottom_line_curve: new fabric.Path('M 0 20',{
            stroke:'rgba(0,100,0, 0.9)',
            strokeWidth:2,
            selectable:false
        }),

        left_line_curve: new fabric.Path('M 0 20',{
            stroke:'rgba(0,100,0, 0.9)',
            strokeWidth:2,
            selectable:false
        }),

        right_line_curve: new fabric.Path('M 0 20',{
            stroke:'rgba(0,100,0, 0.9)',
            strokeWidth:2,
            selectable:false
        }),


    };

    function sanitizePosition() {
        var r = options.restriction;
        zone.left = Math.max(zone.left, r.left);
        zone.top = Math.max(zone.top, r.top);
        zone.right = Math.min(zone.right, r.right);
        zone.bottom = Math.min(zone.bottom, r.bottom);
        if ( zone.right - zone.left < r.min_width) {
            zone.right += r.min_width - (zone.right - zone.left);
        }
        if ( zone.right - zone.left > r.max_width) {
            zone.right -= (zone.right - zone.left) - r.max_width;
        }
        if ( zone.right > r.right ) {
            zone.left -= zone.right - r.right;
            zone.right = r.right;
        }
        if ( zone.left < r.left ) {
            zone.left = r.left;
        }
    }

    function updateEverything(forced_crop) {
        sanitizePosition();
        updatePositions(forced_crop);
    }

    function redraw(forced_crop) {
        // Forced crop likely not needed, but could be used when getting params from backend.
        // Since this might be called from inside the slide-callback
        // we must delay the rendering, so the value is updated.
        setTimeout(function() {
            updateEverything(forced_crop);
            canvas.renderAll();
            $("#zone_json").val(JSON.stringify(zone));
            // console.log("new zone val: ", $("#zone_json").val());
        }, 0);
    }

    for(var i in objs) {
        canvas.add(objs[i]);
    }

    // Needed so initial values on sliders are correct.
    sanitizePosition();
    checkValidity();
    var $offset, $height, $width;
    function setup_dom(reinitialization) {
        if (typeof reinitialization != "undefined" && reinitialization===true) {
            /* The reinitialization might not be needed if sliders can be set correctly on backend call for new zone"*/
            self.find(".ui-slider").parent(".ctrlHolder").remove();
        }
        $offset = addSlider("Counting area offset", "zone_offset", {
            min: res(r.top + r.min_height/2),
            max: res(r.bottom - r.min_height/2),
            value: res(z.top + z.bottom) / 2,
            slide: function(event, ui) {
                var $this = $(this);
                var old = $this.slider("value");
                var height = unres($height.slider("value"));
                var range = r.bottom - r.top - height;
                var new_top = unres(ui.value) - height/2;
                var new_bottom = unres(ui.value) + height/2;
                var valid_move = new_top >= r.top && new_bottom <= r.bottom;

                if (!valid_move) {
                    if ( new_top < r.top ) {
                        $this.slider("value", res(r.top) + $height.slider("value")/2);
                    } else {
                        $this.slider("value", res(r.bottom) - $height.slider("value")/2);
                    }
                    ui.value = $this.slider("value");
                    new_top = unres(ui.value) - height/2;
                    new_bottom = unres(ui.value) + height/2;
                }

                z.top = new_top;
                z.bottom = new_bottom;

                redraw();
                return valid_move;
            }
        });

        $height = addSlider("Counting area height", "zone_height", {
            min: res(r.min_height),
            max: res(r.max_height),
            value: [res(z.bottom) - res(z.top)],
            slide: function (event, ui) {
                var $this = $(this);
                var old  = $this.slider("value");
                var c = (z.bottom + z.top) / 2;

                z.bottom = c + unres(ui.value) / 2;
                z.top = c - unres(ui.value) / 2;

                if ( z.bottom > r.bottom ) {
                    z.top -= z.bottom - r.bottom;
                    z.bottom = r.bottom;
                }

                if ( z.top < r.top ) {
                    z.bottom += r.top - z.top;
                    z.top = r.top;
                }
                c = (z.bottom + z.top) / 2;
                if ($offset.slider("value") != res(c)) {
                    $offset.slider("value", res(c));
                }
                var height = (z.bottom - z.top) / 2;

                redraw();
            }
        });
        if(r.fix_height) {
            $height.parent().hide();
            $("#zone_height_desc").hide();
        }
        $width = addSlider("Counting area width", "zone_width", {
            min: res(r.left),
            max: res(r.right),
            values: [res(z.left), res(z.right)],
            range: true,
            slide: function(event, ui){
                var $this = $(this);
                var old = $(this).slider("values");
                var vals = ui.values;
                var left = unres(vals[0]);
                var right = unres(vals[1]);
                var width = unres(vals[1] - vals[0]);
                if ( width < r.min_width){
                    if ( old[0] == vals[0] ) {
                        if ( old[0] > res(r.left) ) {
                            $this.slider("values", [old[0] + vals[1] -  old[1], vals[1]]);
                        } else {
                            $this.slider("values", [res(r.left), res(r.left + r.min_width)]);
                        }
                    } else {
                        if ( old[1] < res(r.right) ) {
                            $this.slider("values", [vals[0], old[1] + vals[0] -  old[0]]);
                        } else {
                            $this.slider("values", [res(r.right - r.min_width), res(r.right)]);
                        }
                    }
                    z.left = unres($width.slider("values")[0]);
                    z.right = unres($width.slider("values")[1]);
                    redraw();
                    return false;
                }
                if ( width > r.max_width) {
                    if ( old[0] == vals[0] ) {
                        $this.slider("values", [old[0] + vals[1] -  old[1], vals[1]]);
                    } else {
                        $this.slider("values", [vals[0], old[1] + vals[0] -  old[0]]);
                    }
                    z.left = unres($width.slider("values")[0]);
                    z.right = unres($width.slider("values")[1]);
                    redraw();
                    return false;
                }
                z.left = unres($width.slider("values")[0]);
                z.right = unres($width.slider("values")[1]);
                redraw();
            }
        });
    }
    setup_dom();

    function calculateHeadArea(zone) {
        /*
         * Calculates head area from floor area.
         */
        headArea =  $.extend({}, zone); //Clone zone to headArea
        headArea.left = 0.5 - (0.5 - headArea.left) * installation_height / (installation_height - 140);
        headArea.right = 0.5 + (headArea.right - 0.5) * installation_height / (installation_height - 140);
        headArea.top = 0.5 - (0.5 - headArea.top) * installation_height / (installation_height - 140);
        headArea.bottom = 0.5 + (headArea.bottom - 0.5) * installation_height / (installation_height - 140);
        return headArea;
    }

    // Some canvas traits for example the path of a Path object doesn't like to be updated.
    // Use this function to replace the entire object.
    function update_canvas(objs, name, new_object) {
        canvas.remove(objs[name]);
        objs[name] = new_object;
        canvas.add(objs[name]);
    }

    function point_to_path(point) {
        return ' ' + point['x']*width + ' ' + point['y']*height + ' '
    }

    // Sets currentArrow to the correct one depending on the counting area's state
    function setCurrentArrowFromDirections() {
        dirArray = [zone.top_dir, zone.right_dir, zone.bot_dir, zone.left_dir];

        // Try to find the correct arrow in relation to the current state of the counting area.
        //
        // Returns: An object containing:
        //
        //      count: count   - the number of times the image needs to rotate 90 degrees to match
        //      mirror: mirror - boolean saying if the image needs to be mirrored in x-axis
        //
        // or undefined if no match was found.
        function deliverArrowData(arrowRotation) {

            // Rotate the arrow the correct number of times to match the counting area.
            //
            // Returns:
            //
            //      counts - number of times the image needs to rotate
            //
            // or undefined indicating that the image needs to be mirrored.
            function rotateAndCheck(arrowRotation) {

                // Check if counting area directions and arrow directions are the same
                function checkMatch() {
                    for (var i = 0; i < dirArray.length; i++) {
                        if (dirArray[i] !== arrowRotation[i]) {
                            return false;
                        }
                    }
                    return true;
                }

                var match = false;
                for (var count = 0; count < 4; count++) {
                    match = checkMatch();
                    if (!match) {

                        // Rotate the arrow by shifting array elements one step to the right
                        var lastElement = arrowRotation.pop();
                        arrowRotation.splice(0, 0, lastElement);
                    } else {
                        return count;
                    }
                }
                return undefined;
            }

            var mirror = false;
            var count = rotateAndCheck(arrowRotation);

            // If no match the first time - mirror it
            if (count === undefined) {
                mirror = true;
                var copy = arrowRotation.slice();
                var temp = copy[1];
                copy[1] = copy[3];
                copy[3] = temp;
                count = rotateAndCheck(copy);
            }
            return count === undefined ? undefined : {count: count, mirror: mirror};
        }

        // Find the correct arrow
        Object.keys(arrowImageData).some(function (key) {
            var shape = arrowImageData[key]["shape"].slice();
            var arrowData = deliverArrowData(shape);
            if (arrowData !== undefined) {
                currentArrow = arrowImageData[key]["obj"];
                currentArrow.angle = arrowImageData[key]["angle"] + 90 * arrowData.count;
                currentArrow.flipY = arrowData.mirror;
                canvas.add(currentArrow);
                return true;
            }
        });
    }

    function updatePositions(forced_crop) {

        /*
         * The floor area have the same relation to z, as the head area have to the crop zone,
         * assuming that they all were defined in the same coordinate system.
         *
         * floor.width = z.width - 2*zone_trigger_limit*min(z.width, z.height)
         * floor.heigh = z.heigh - 2*zone_trigger_limit*min(z.width, z.height)
         *
         * This results in the counting area corresponding to the head area, with two exceptions
         * that are handled differently in the backend relative to the front end.
         *
         * 1. If the counting area is to far up or down, the backend will translate it so that it
         * will fit in the image, if possible.
         *
         * 2. If the floor area is not contained in the head area, the counting area is expanded to
         * cover the head area.
         *
         */

        var floorArea = $.extend({}, z);

        // floor width/height uses absolute values
        var floorWidth = Math.abs(floorArea.right - floorArea.left) * width
        var floorHeight = Math.abs(floorArea.top - floorArea.bottom) * height

        // margin computed in absolute values
        var margin = Math.min(floorWidth, floorHeight) * params.opt.zone_trigger_limit

        // compute relative margins for width and height
        var relativeWidthMargin  = margin / width
        var relativeHeightMargin = margin / height

        if (floorArea.left_dir != 'none') {
            floorArea.left += relativeWidthMargin
        }
        if (floorArea.right_dir != 'none') {
            floorArea.right -= relativeWidthMargin
        }
        if (floorArea.top_dir != 'none') {
            floorArea.top += relativeHeightMargin
        }
        if (floorArea.bot_dir != 'none') {
            floorArea.bottom -= relativeHeightMargin
        }

        if (forced_crop !== undefined) {
            headArea = forced_crop;
        } else {
            headArea = calculateHeadArea(floorArea);
        }

        var headAreaLeftBottom = {x:headArea.left, y:headArea.bottom }
        var headAreaLeftTop = {x:headArea.left, y:headArea.top}
        var headAreaRightBottom = {x:headArea.right, y:headArea.bottom}
        var headAreaRightTop = {x:headArea.right, y:headArea.top}

        var floorAreaLeftBottom = {x:floorArea.left, y:floorArea.bottom}
        var floorAreaLeftTop = {x:floorArea.left, y:floorArea.top}
        var floorAreaRightBottom = {x:floorArea.right, y:floorArea.bottom}
        var floorAreaRightTop = {x:floorArea.right, y:floorArea.top}

        var top_path =
              'M' + point_to_path(headAreaLeftTop)
            + 'L' + point_to_path(floorAreaLeftTop)
            + 'L' + point_to_path(floorAreaRightTop)
            + 'L' + point_to_path(headAreaRightTop)
            + 'L' + point_to_path(headAreaLeftTop)

        var bottom_path =
              'M' + point_to_path(headAreaLeftBottom)
            + 'L' + point_to_path(floorAreaLeftBottom)
            + 'L' + point_to_path(floorAreaRightBottom)
            + 'L' + point_to_path(headAreaRightBottom)
            + 'L' + point_to_path(headAreaLeftBottom)

        var left_path =
              'M' + point_to_path(headAreaLeftBottom)
            + 'L' + point_to_path(floorAreaLeftBottom)
            + 'L' + point_to_path(floorAreaLeftTop)
            + 'L' + point_to_path(headAreaLeftTop)
            + 'L' + point_to_path(headAreaLeftBottom)

        var right_path =
              'M' + point_to_path(headAreaRightBottom)
            + 'L' + point_to_path(floorAreaRightBottom)
            + 'L' + point_to_path(floorAreaRightTop)
            + 'L' + point_to_path(headAreaRightTop)
            + 'L' + point_to_path(headAreaRightBottom)

        function get_color_from_direction(dir){
            var color = "rgba(120, 120, 120, 1.0)"; // Neutral color
            var color_dark = "rgba(60, 60, 60, 1.0)"; // Dark neutral color
            if(dir === "in"){
                color = "rgba(124, 210, 74, 1.0)";
                color_dark = "rgba(5, 40, 1, 1.0)";
            }
            else if(dir === "out"){
                color = "rgba(36, 150, 255, 1.0)";
                color_dark = "rgba(20, 40, 70, 1.0)";
            }

            return {normal: color, dark: color_dark};
        }

        // Map direction to zone color
        var top_color = get_color_from_direction(zone.top_dir);
        var right_color = get_color_from_direction(zone.right_dir);
        var bot_color = get_color_from_direction(zone.bot_dir);
        var left_color = get_color_from_direction(zone.left_dir);

        var stroke_width = 2
        var box_opacity = 0.4

        if(currentArrow){
            var xOffset = (floorArea.right - floorArea.left)/2;
            var yOffset = (floorArea.top - floorArea.bottom)/2
            currentArrow.left = (floorArea.left + xOffset) * width;
            currentArrow.top = (floorArea.top - yOffset) * height;
        }

        canvas.renderAll();
        update_canvas(objs, 'left_line_curve', new fabric.Path(left_path,{
            top: Math.min(headArea.top, floorArea.top)* height,
            left: Math.min(headArea.left, floorArea.left)* width,

            strokeWidth:stroke_width,
            opacity: box_opacity,
            fill: left_color.normal,
            stroke: left_color.dark,
        }));

        objs.left_line_curve.setGradient('fill', {
            'x1': 0,
            'y1': 0,
            'x2': Math.abs(floorArea.left - headArea.left) * width,
            'y2': 0,
            'colorStops': {
                0: left_color.normal,
                1: left_color.dark
            }
        });
        update_canvas(objs, 'right_line_curve', new fabric.Path(right_path,{
            top: Math.min(headArea.top, floorArea.top)* height,
            left: Math.min(headArea.right, floorArea.right)* width,
            strokeWidth:stroke_width,
            opacity: box_opacity,
            fill: right_color.normal,
            stroke: right_color.dark,
        }));

        objs.right_line_curve.setGradient('fill', {
            'x1': Math.abs(floorArea.right - headArea.right) * width,
            'y1': 0,
            'x2': 0,
            'y2': 0,
            'colorStops': {
                0: right_color.normal,
                1: right_color.dark
            }
        });

        update_canvas(objs, 'top_line_curve', new fabric.Path(top_path,{
            top: Math.min(headArea.top, floorArea.top)* height,
            left: Math.min(headArea.left, floorArea.left)* width,
            strokeWidth:stroke_width,
            opacity: box_opacity,
            fill: top_color.normal,
            stroke: top_color.dark,
        }));

        objs.top_line_curve.setGradient('fill', {
            'x1': 0,
            'y1': 0,
            'x2': 0,
            'y2': Math.abs(floorArea.top - headArea.top) * height,
            'colorStops': {
                0: top_color.normal,
                1: top_color.dark
            }
        });

        update_canvas(objs, 'bottom_line_curve', new fabric.Path(bottom_path,{
            top: Math.min(headArea.bottom, floorArea.bottom)* height,
            left: Math.min(headArea.left, floorArea.left)* width,

            strokeWidth:stroke_width,
            opacity: box_opacity,
            fill: bot_color.nomal,
            stroke: bot_color.dark,
        }));

        objs.bottom_line_curve.setGradient('fill', {
            'x1': 0,
            'y1': Math.abs(floorArea.bottom - headArea.bottom) * height,
            'x2': 0,
            'y2': 0,
            'colorStops': {
                0: bot_color.normal,
                1: bot_color.dark
            }
        });
    }

    // Trigger slide on all sliders.
    function trigger_slides() {
        [$width, $offset, $height].forEach(function(slider) {
            var $slider = $(slider),
                ui = {};

            ui.value = $slider.slider("value");
            ui.values = $slider.slider("values");
            /*
            ui.value = res(Math.random());
            ui.values = [res(0.2 + 0.6 * Math.random()), res(0.8)];
            */
            $slider.slider("option", "slide").call($slider, {}, ui); // Always a bit of magic needed to dynamically set sliders. This was found in StackOverflow if i'm not mistaken.
        });
    }

    // Checks direction validity such that there is at least
    // one "in" and one "out" direction.
    function checkValidity(){
        $directions = $("select[name$='_direction']");
        var dir_in = false;
        var dir_out = false;
        for(i in $directions){
            if($directions[i].value === 'in') dir_in = true;
            else if($directions[i].value === 'out') dir_out = true;
        }
        if(!(dir_in && dir_out)){
            $("#hidden_validation").addClass("validation-error");
            $("#hidden_validation").show();
        }else{
            $("#hidden_validation").removeClass("validation-error");
            $("#hidden_validation").hide();
        }
    }

    if ( !options.view_only ) {
        /*
         * View only is set on liveview for example, then this does not need to be run.
         */
        trigger_slides();
        $("#top_direction, #right_direction, #bot_direction, #left_direction").change(function() {
            var value = $(this).val();
            switch($(this).context.id){
                case "top_direction":
                    zone.top_dir = value
                    break;
                case "right_direction":
                    zone.right_dir = value;
                    break;
                case "bot_direction":
                    zone.bot_dir = value;
                    break;
                case "left_direction":
                    zone.left_dir = value;
                    break;
            }
            checkValidity();
            canvas.remove(currentArrow);
            setCurrentArrowFromDirections();
            redraw();
        });
    }

    redraw();
    if ( options.view_only ) {
        redraw(); //Everything doesn't render otherwise.. unclear why...
    }

    // Necessary due to asynchronous arrow image loading
    $(window).load(function () {
        setCurrentArrowFromDirections();
        redraw();
    });
};