/*global $, window, params*/
$(function () {
    "use strict";
    function getImageVapix(cameraId) {
        return new Promise((resolve, reject) => {
            const xhr = new XMLHttpRequest();
            xhr.open("GET", "/axis-cgi/jpg/image.cgi?camera=" + cameraId, true);
            xhr.responseType = "arraybuffer";
            xhr.onload = () => resolve({'status': xhr.status, 'response': xhr.response});
            xhr.onerror = () => reject(xhr.statusText);
            xhr.send();
        });
    };

    function getTextData(url) {
        return new Promise((resolve, reject) => {
            const xhr = new XMLHttpRequest();
            xhr.open("GET", url, true);
            xhr.responseType = 'text';
            xhr.onload = () => resolve({'status': xhr.status, 'response': xhr.response});
            xhr.onerror = () => reject(xhr.statusText);
            xhr.send();
        });
    };

    function isUserAdmin() {
        return new Promise((resolve, reject) => {
            const xhr = new XMLHttpRequest();
            xhr.open("GET", "/stereo/whoami", true);
            xhr.responseType = "json";
            xhr.onload = () => resolve(xhr.response);
            xhr.onerror = () => reject(xhr.statusText);
            xhr.send();
        });
    };

    function getServerReport() {
        return isUserAdmin().then((data) => {
            if (data.user === "admin") {
                return getTextData("/axis-cgi/serverreport.cgi");
            } else {
                return Promise.resolve({'status': 401, 'response': undefined});
            }
        });
    };

    function exportCountData() {
        return getTextData("/stereo/people-counter/export.csv?export-method=on&date=all&resolution=60&method=open")
    };

    function getSystemLogs() {
        return isUserAdmin().then((data) => {
            if (data.user === "admin") {
                return getTextData("/axis-cgi/systemlog.cgi");
            } else {
                return Promise.resolve({'status': 401, 'response': undefined});
            }
        });
    };

    function getDebugZip(download_url) {
        return new JSZip.external.Promise(function (resolve, reject) {
            JSZipUtils.getBinaryContent(download_url, function(err, data) {
                if (err) {
                    reject(err);
                } else {
                    resolve(data);
                }
            });
        }).then(function(data) {
            return JSZip.loadAsync(data);
        });
    };

    var $debugForm = $("#maintenance-form-record-debug-data");
    $debugForm.submit(function (e) {
        var record_time = $(this).find("#debug_record_time").val(),
            $button = $("#button-debug-download");
        e.preventDefault();
        $.ajax({
            type: "GET",
            url: "debug_download_prepare",
            cache: false,
            data: {
                record_time: record_time,
            },
            beforeSend: function () {
                $button.prop("disabled", true);
                $button.addClass("disabled");
                $button.addLoader("light");
            },
            success: function (data) {
                if (data.status === "success") {
                    Promise.all([getImageVapix(1),
                                 getImageVapix(2),
                                 getDebugZip(data.download_url),
                                 getServerReport(),
                                 exportCountData(),
                                 getSystemLogs()])
                    .then((results) => {
                        let img1 = results[0];
                        let img2 = results[1];
                        let zip = results[2];
                        let report = results[3];
                        let countData = results[4];
                        let syslog = results[5];
                        if (syslog.status === 200) {
                            zip.file("logs/syslog.txt", syslog.response);
                        }
                        if (report.status === 200) {
                            zip.file("serverreport.txt", report.response);
                        }
                        if (img1.status === 200) {
                            zip.file("images/camera1.jpg", img1.response, {binary: true});
                        }
                        if (img2.status === 200) {
                            zip.file("images/camera2.jpg", img2.response, {binary: true});
                        }
                        if (countData.status === 200) {
                            zip.file("count_data.csv", countData.response);
                        }
                        return zip.generateAsync({type: "base64"}).then((base64) => {
                            $button.prop("disabled", false);
                            $button.removeClass("disabled");
                            $button.removeLoader();
                            location.href = "data:application/zip;base64," + base64;
                        }).catch((err) => {
                            throw new Error(err);
                            });
                    }).catch((err) => {
                        alert("Failed to create debug archive. Reason: " + err);
                        $button.prop("disabled", false);
                        $button.removeClass("disabled");
                        $button.removeLoader();
                    });
                }
            },
            error: function (error) {
                alert(error.responseJSON.message);
                $button.prop("disabled", false);
                $button.removeClass("disabled");
                $button.removeLoader();
            }
        });
    });
    $(".hasNotice").click(function (){
        var id = $(this).attr("id"),
            $parentDiv = $(this).parents(".maintenaceInner"),
            $notices = $parentDiv.find(".showOnClick.for-" + id);
        $notices.slideDown();
    });
    $("#button-restart-camera").click(function(e) {
        e.preventDefault();
        if ( confirm("This might take several minutes, do you want to continue?") ) {
            // Not tested with proxy
            $.when($.getStartTime()).done(function(ref_start_time){
                $.ajax("/axis-cgi/admin/restart.cgi");
                $.waitForApp(function(){location.reload();}, 10000, ref_start_time);
            });
        }
    });
    $("#button-restart-application").click(function(e) {
        e.preventDefault();
        if ( confirm("This might take a minute, do you want to continue?") ) {
            $.restartAndWait(function(){location.reload();});
        }
    });

    $("#button-registration-page").click(function(e) {
        window.location.href = "/stereo/registration.html";
        e.preventDefault();
    });

    $("#goto-calibration").click(function(e) {
        window.location.href = "/stereo/calibration.html";
        e.preventDefault();
    });

    $("#button-show-logs").click(function(e) {
        window.open("/logs/stereo.log");
        e.preventDefault();
    });

    $("#button-clear-data").click(function(e) {
        if ( confirm("This will remove all historical data, do you want to continue?") ) {
            $("#overlay").show();
            $.post("/stereo/data/clear", {}, function(data) {
                if (data && data['status'] == 'CM_OK') {
                    console.log("Successfully cleared historical data");
                } else {
                    if ( data && data['message'] ) {
                        alert("Could not clear data: " + data['message']);
                    } else {
                        alert("Could not clear data.");
                    }
                }
            }).fail(function() {
                alert("Could not clear data");
            });
            /*
            / As the execution time may be so small that the overlay does not show,
            / it might seem that the button had no effect. A minor delay of half a second
            / is added so a feedback is always given to the user.
            */
            $("#overlay").delay(500).hide(0);
        }
        e.preventDefault();
    });

    $("#button-analysis-mode").click(function(e) {
        e.preventDefault();
        if ( confirm("This might take a minute, do you want to continue?") ) {
            $.when($.getStartTime(), $.get("/stereo/params.json")).done(function(startup_time, json_data) {
                // For reasons unclear to me jquery.when returns each response as an array where index 0 is the response body.
                let analysis_mode = json_data[0].analysis_mode;
                if (analysis_mode) {
                    let msg = 'Could not disable analysis mode';
                } else {
                    let msg = 'Could not enable analysis mode.';
                }
                $.post("/stereo/analysis_mode", {"value": !analysis_mode}, function(data) {
                    if (data && data['status'] == 202) {
                        $.waitForApp(function(){location.reload();}, undefined, startup_time[0]);
                    } else {
                        alert(msg);
                    }
                }).fail(function() {
                   console.log(msg);
                });
            });
        }
    });

    $("#share-data-checkbox").change(function() {
        $.setShareData(this.checked, function() {
            location.reload();
        });
    });

    window.addEventListener("load", function(e) {
        $.get("/stereo/params.json").then(function(data) {
            if (data.analysis_mode) {
                $("#button-analysis-mode").text("Disable analysis mode");
            } else {
                $("#button-analysis-mode").text("Enable analysis mode");
            }
            if (data.share_data.enabled != false) {
                $("#share-data-checkbox").prop('checked', true)
            }
        });
    });

    function createErrorCallback(message) {
        if (typeof message === 'undefined') {
            message = "An unknown error has occurred"
        }
        var func = function() {
            console.log(message);
        }
        return func;
    }
});
